# ПРИМЕР НАСТРОЙКИ МОДУЛЯ:
                                                #
from pyiArduinoI2Ctds import *                  #   Подключаем библиотеку для работы с TDS/EC-метром I2C-flash.
tds = pyiArduinoI2Ctds(0x09)                    #   Объявляем объект tds  для работы с функциями и методами библиотеки pyiArduinoI2Ctds, указывая адрес модуля на шине I2C.
                                                #   Если объявить объект без указания адреса (pyiArduinoI2Ctds tds ), то адрес будет найден автоматически.
tds.setFrequency(  2000)                        #   Устанавливаем частоту переменного тока для измерений от 50 до 5000 Гц. Значение сохраняется в энергонезависимую память.
tds.setKnownTDS (1, 500)                        #   Устанавливаем концентрацию 1 раствора для калибровки модуля с кнопки.  Значение сохраняется в энергонезависимую память.
tds.setKnownTDS (2,1500)                        #   Устанавливаем концентрацию 2 раствора для калибровки модуля с кнопки.  Значение сохраняется в энергонезависимую память.
tds.setKt       (  0.02)                        #   Устанавливаем температурный коэффициент жидкости от 0,0000 до 6,5535.
tds.setKp       (  0.50)                        #   Устанавливаем коэффициент пересчёта от 0,01 до 2,55.
tds.set_T       ( 25.00)                        #   Устанавливаем опорную температуру жидкости от 0,00 до 63,75 °С.



# ПРИМЕР ПОЛУЧЕНИЯ ДАННЫХ ПО ШИНЕ I2C:
                                                  #
from time import sleep
from pyiArduinoI2Ctds import *                    #   Подключаем библиотеку для работы с TDS/EC-метром I2C-flash.
tds = pyiArduinoI2Ctds(0x09)                      #   Объявляем объект tds  для работы с функциями и методами библиотеки pyiArduinoI2Ctds, указывая адрес модуля на шине I2C.
                                                  #   Если объявить объект без указания адреса (pyiArduinoI2Ctds tds ), то адрес будет найден автоматически.
while True:                                       #
     tds.set_t(22.5)                              #   Указываем текущую температуру жидкости.
     print("Ro="          , end='')               #
     print(tds.getRo()    , end='')               #   Выводим общее измеренное сопротивление.
     print("Ом, S="       , end='')               #
     print(tds.get_S()    , end='')               #   Выводим измеренную удельную электропроводность жидкости.
     print("мкСм/см, EC=" , end='')               #
     print(tds.getEC()    , end='')               #   Выводим удельную электропроводность жидкости приведённую к опорной температуре.
     print("мкСм/см, TDS=", end='')               #
     print(tds.getTDS()   , end='')               #   Выводим количество растворённых твёрдых веществ в жидкости.
     print(" мг/л\r\n"    , end='')               #
     sleep(1)                                     #



cdef extern from "<stdint.h>" nogil:
    ctypedef signed char int8_t
    ctypedef signed short int16_t
    ctypedef signed int int32_t
    ctypedef signed long int64_t
    ctypedef unsigned char uint8_t
    ctypedef unsigned short uint16_t
    ctypedef unsigned int uint32_t
    ctypedef unsigned long long uint64_t

cdef extern from "iarduino_I2C_TDS.cpp":
    pass

cdef extern from "iarduino_I2C_TDS.h":
    cdef cppclass iarduino_I2C_TDS:
        iarduino_I2C_TDS() except +
        iarduino_I2C_TDS(unsigned char) except +
        bint begin()
        bint changeAddress(unsigned char)
        bint reset()
        unsigned char getAddress()
        unsigned char getVersion()

        bint getPullI2C()
        bint setPullI2C(bint)

        uint16_t getFrequency()
        bint setFrequency(uint16_t)
        bint setCalibration(uint8_t, uint16_t)
        uint16_t getKnownTDS(uint8_t)
        bint setKnownTDS(uint8_t, uint16_t)
        float getKa()
        bint setKa(float)
        float getKb()
        bint setKb(float)
        float getKt()
        bint setKt(float)
        float getKp()
        bint setKp(float)
        bint set_t(float)
        float get_T()
        bint set_T(float)
        uint32_t getRo()
        float getVout()
        uint16_t get_S()
        uint16_t getEC()
        uint16_t getTDS()



# distutils: language = c++
# cython: language_level = 3

from iarduino_I2C_TDS cimport iarduino_I2C_TDS
#from time import sleep

DEF_CHIP_ID_FLASH =  0x3C
DEF_CHIP_ID_METRO =  0xC3

# Адреса регистров модуля:
REG_FLAGS_0          =  0x00
REG_BITS_0           =  0x01
REG_FLAGS_1          =  0x02
REG_BITS_1           =  0x03
REG_MODEL            =  0x04
REG_VERSION          =  0x05
REG_ADDRESS          =  0x06
REG_CHIP_ID          =  0x07
REG_TDS_FREQUENCY_L  =  0x08
REG_TDS_FREQUENCY_H  =  0x09
REG_TDS_KNOWN_TDS    =  0x0A
REG_TDS_KNOWN_TDS_1  =  0x0C
REG_TDS_KNOWN_TDS_2  =  0x0E
REG_TDS_CALIBRATION  =  0x10
REG_TDS_Ka           =  0x11
REG_TDS_Kb           =  0x14
REG_TDS_Kt           =  0x16
REG_TDS_Kp           =  0x18
REG_TDS_t            =  0x19
REG_TDS_T            =  0x1A
REG_TDS_Ro           =  0x1B
REG_TDS_Vout         =  0x1E
REG_TDS_S            =  0x20
REG_TDS_EC           =  0x22
REG_TDS_TDS          =  0x24

# Позиция битов и флагов:
TDS_BIT_CALC_2       =  0x02
TDS_BIT_CALC_1       =  0x01

NO_BEGIN = 1

cdef class pyiArduinoI2Ctds:
    cdef iarduino_I2C_TDS c_module

    def __cinit__(self, address=None, auto=None):

        if address is not None:

            self.c_module = iarduino_I2C_TDS(address)

            if auto is None:
                #sleep(.5)
                if not self.c_module.begin():

                    print("ошибка инициализации модуля.\n"
                          "Проверьте подключение и адрес модуля,"
                          " возможно не включен интерфейс I2C.\n"
                          " Запустите raspi-config и включите интерфейс"
                          ", инструкция по включению:"
                          " https://wiki.iarduino.ru/page/raspberry-i2c-spi/")

        else:

            self.c_module = iarduino_I2C_TDS()

            if auto is None:
                #sleep(.5)
                if not self.c_module.begin():

                    print("ошибка инициализации модуля.\n"
                          "Проверьте подключение и адрес модуля, "
                          " возможно не включен интерфейс I2C.\n"
                          " Запустите raspi-config и включите интерфейс"
                          ", инструкция по включению:"
                          " https://wiki.iarduino.ru/page/raspberry-i2c-spi/")

    def begin(self):
        return self.c_module.begin()

    def changeAddress(self, unsigned char newAddr):
        return self.c_module.changeAddress(newAddr)

    def reset(self):
        return self.c_module.reset()

    def getAddress(self):
        return self.c_module.getAddress()

    def getVersion(self):
        return self.c_module.getVersion()

    def getPullI2C(self):
        return self.c_module.getPullI2C()

    def setPullI2C(self, flag=None):
        if flag is None:
            return self.c_module.setPullI2C(True)
        return self.c_module.setPullI2C(flag)

    def getFrequency(self):
        return self.c_module.getFrequency()

    def setFrequency(self, freq):
        return self.c_module.setFrequency(freq)

    def setCalibration(self, num, tds):
        return self.c_module.setCalibration(num, tds)

    def getKnownTDS(self, num):
        return self.c_module.getKnownTDS(num)

    def setKnownTDS(self, num, tds):
        return self.c_module.setKnownTDS(num, tds)

    def getKa(self):
        return self.c_module.getKa()

    def setKa(self, val):
        return self.c_module.setKa(val)

    def getKb(self):
        return self.c_module.getKb()

    def setKb(self, val):
        return self.c_module.setKb(val)

    def getKt(self):
        return self.c_module.getKt()

    def setKt(self, val):
        return self.c_module.setKt(val)

    def getKp(self):
        return self.c_module.getKp()

    def setKp(self, val):
        return self.c_module.setKp(val)

    def get_T(self):
        return self.c_module.get_T()

    def set_t(self, val):
        return self.c_module.set_t(val)

    def set_T(self, val):
        return self.c_module.set_T(val)

    def getRo(self):
        return self.c_module.getRo()

    def get_S(self):
        return self.c_module.get_S()

    def getEC(self):
        return self.c_module.getEC()

    def getTDS(self):
        return self.c_module.getTDS()

    def getVout(self):
        return self.c_module.getVout()



# ПРИМЕР СМЕНЫ АДРЕСА МОДУЛЯ:

# Подключаем библиотеку для работы с датчиком температуры и влажности
from pyiArduinoI2Ctds import *
import sys

# Объявляем объект module для работы с функциями и методами библиотеки pyiArduinoI2Ctds.
# Если при объявлении объекта указать адрес, например, module(0x0B),
# то пример будет работать с тем модулем, адрес которого был указан.
module = pyiArduinoI2Ctds(None, NO_BEGIN)

# Если сценарию не были переданы аргументы
if len(sys.argv) < 2:
    # Назначаем модулю адрес (0x07 < адрес < 0x7F).
    newAddress = 0x09

# Иначе
else:
    # Новый адрес - первый аргумент
    newAddress = int(sys.argv[1])

# Если датчик найден
if module.begin():
    print("Найден датчик %#.2x" % module.getAddress())

    # Если адрес удалось изменить
    if module.changeAddress(newAddress):
            print("Адрес изменён на %#.2x" % module.getAddress())

    else:
            print("Адрес не изменён!")

else:
    print("Датчик не найден!")



# ПРИМЕР ПОЛУЧЕНИЯ НАСТРОЕК МОДУЛЯ:
                                              #
from pyiArduinoI2Ctds import *                #   Подключаем библиотеку для работы с TDS/EC-метром I2C-flash.
tds = pyiArduinoI2Ctds(0x09)                  #   Объявляем объект tds  для работы с функциями и методами библиотеки pyiArduinoI2Ctds, указывая адрес модуля на шине I2C.
                                              #   Если объявить объект без указания адреса (pyiArduinoI2Ctds tds ), то адрес будет найден автоматически.
print("F    = "         , end='')             #
print(tds.getFrequency(), end='')             #   Выводим частоту переменного тока используемую для измерений от 50 до 500 Гц.
print("Гц.\r\nTDS1 = "  , end='')             #
print(tds.getKnownTDS(1), end='')             #   Выводим требуемую концентрацию 1 раствора для калибровки модуля кнопкой от 0 до 10'000 ppm.
print("ppm.\r\nTDS2 = " , end='')             #
print(tds.getKnownTDS(2), end='')             #   Выводим требуемую концентрацию 2 раствора для калибровки модуля кнопкой от 0 до 10'000 ppm.
print("ppm.\r\nKt   = " , end='')             #
print(tds.getKt()     ,4, end='')             #   Выводим температурный коэффициент жидкости от 0,0000 до 6,5535.
print("\r\nKp   = "     , end='')             #
print(tds.getKp()     ,2, end='')             #   Выводим коэффициент пересчёта от 0,01 до 2,55.
print("\r\nT    = "     , end='')             #
print(tds.get_T()     ,2, end='')             #   Выводим опорную температуру жидкости от 0,00 до 63,75 °С.
print("°С.\r\nKa   = "  , end='')             #
print(tds.getKa()     ,2, end='')             #   Выводим множитель степеной функции от 0,01 до 167'772.15.
print(".\r\nKb   = "    , end='')             #
print(tds.getKb()     ,3, end='')             #   Выводим степень степенной функции от -0,010 до -65.535.
print(".\r\n"           , end='')             #



# ПРИМЕР КАЛИБРОВКИ МОДУЛЯ:
                                                  #
from pyiArduinoI2Ctds import *                    #   Подключаем библиотеку для работы с TDS/EC-метром I2C-flash.
tds = pyiArduinoI2Ctds(0x09)                      #   Объявляем объект tds  для работы с функциями и методами библиотеки pyiArduinoI2Ctds, указывая адрес модуля на шине I2C.
                                                  #   Если объявить объект без указания адреса (pyiArduinoI2Ctds tds ), то адрес будет найден автоматически.
val_t = 23.0                                      #   Температура калибровочных жидкостей.
                                                  #
tds.set_t(val_t)                                  #   Указываем температуру жидкости в °С.
print("ОПУСТИТЕ ДАТЧИК В 1 РАСТВОР", end='')      #
print(" (500 ppm). ")                             #
sleep(60)                                         #   Ждём 1 минуту.
print("КАЛИБРОВКА...")                            #
tds.setCalibration(1, 500)                        #   Выполняем 1 стадию калибровки указав известную концентрацию 1 раствора (в примере 1234ppm ).
sleep(10)                                         #   Ждём 10 секунд.
print("СПОЛОСНИТЕ ДАТЧИК.")                       #
sleep(30)                                         #   Ждём 30 секунд.
print("ОПУСТИТЕ ДАТЧИК В 2 РАСТВОР", end='')      #
print(" (1500 ppm). ")                            #
sleep(60)                                         #   Ждём 1 минуту.
print("КАЛИБРОВКА...")                            #
tds.setCalibration(2, 1500 )                      #   Выполняем 2 стадию калибровки указав известную концентрацию 2 раствора (в примере 4321ppm ).
sleep(10)                                         #   Ждём 10 секунд.
print("КАЛИБРОВКА ВЫПОЛНЕНА!")                    #
                                                  #
while True:                                       #
     print("TDS=", end='')                        #
     print(tds.getTDS() )                         #   Выводим количество растворённых твёрдых веществ в жидкости.
     print("мг/л.\r\n")                           #
     sleep(1)                                     #



# Данный пример ищет модули на шине и выводит информацию о них.
                                            #
import smbus                                # Подключаем модуль шины smbus(i2c)
choices = {                                 # Создаём список устройств
0x01: "кнопкой",                            # Если значение 1, значит это кнопка.
0x02: "светодиодом",                        # Если значение 2, значит это RGB светодиод.
0x03: "потенциометром",                     # Если значение 3, значит это потенциометр.
0x04: "звукоизлучателем",                   # Если значение 4, значит это звукоизлучатель.
0x05: "датчиком SHT",                       # Если значение 5, значит это датчик владности и температуры.
0x06: "датчиком света",                     # Если значение 6, значит это датчик света.
0x07: "расширителем выводов",               # Если значение 7, значит это датчик света.
0x08: "LED матрицей",                       # Если значение 8, значит это светодиодная матрица.
0x09: "Энкодером",                          # Если значение 9, значит это энкодер.
0x0A: "реле на 2 канала",                   # Если значение A, значит это электромеханическое реле на 2 канала.
0x0B: "реле на 4 канала",                   # Если значение B, значит это твердотельное реле на 4 канала.
0x0C: "силовым ключём 4 N",                 # Если значение C, значит это силовой ключ на 4 N-канала.
0x0D: "силовым ключём 4 P",                 # Если значение D, значит это силовой ключ на 4 P-канала.
0x0E: "силовым ключём 4 N",                 # Если значение E, значит это силовой ключ на 4 N-канала, с измерением тока.
0x0F: "силовым ключём 4 P"                  # Если значение F, значит это силовой ключ на 4 P-канала, с измерением тока.
}                                           #
try:                                        #
  bus = smbus.SMBus(1)                      # Пробуем открыть файл шины i2c
except FileNotFoundError:                   # Если файл не найден
  print("Шина i2c не включена. Запустите"   #
        "`raspi-config` и включите шину.")  # Выводим сообщение в stdout
except:                                     # Если любая другая ошибка
  print("неизвестная ошибка")               # Выводим сообщение в stdout
else:                                       # Если ошибок не найдено
  for i in range(7, 127):                   # Проходим по всем доступным адресам на шине I2C ...
    try:                                    # Пробуем вывести адрес
      bus.write_quick(i)                    # устройства на шину
    except OSError:                         # Если устройства с текущим адресом не на шине
      continue                              # Возвращаемся в начало цикла for со следующим адресом
    except:                                 # Если любая другая ошибка
      print("неизвестная ошибка")           # Выводим сообщение в stdout
    else:                                   # Если устройство найдено
      print("Устройство с адресом"
            " %#.2X" % i, end='')            # Выводим адрес в stdout
      k = bus.read_i2c_block_data(i, 0x04)  # Читаем регистры, начиная с 4
      if k[2]>>1 == i and (k[3]==0xc3
                          or k[3]==0x3c):   # Если значение второго элемента массива k совпадает с адресом устройства, а в третьем элементе хранится идентификатор 0xC3 (модуль Metro), или 3С (модуль Flash), то ...
        print(" является ", end='')         # Выводим текст в stdout
        model = choices.get(k[0],
              "неизвестным модулем")        # Cравниваем модель модуля со списком, если ничего не совпало - записываем в model строку по умолчанию
        print("%s с версией прошивки %d."
               % (model, k[1]))             # Выводим модель и версию в stdout
      else:                                 # Выводим текст, если устройство не опознано
        print(" не опознано")               # или на шине два устроойства с одним адресом
